from importlib.resources import files

OP_ERASE = "erase"
OP_CHECK = "check"

# The "plain" variants are synonyms for AS
ATF_1502 = "ATF1502"
ATF_1504 = "ATF1504"
ATF_1508 = "ATF1508"

ATF_1502AS = "ATF1502AS"
ATF_1504AS = "ATF1504AS"
ATF_1508AS = "ATF1508AS"
ATF_1502ASV = "ATF1502ASV"
ATF_1504ASV = "ATF1504ASV"
ATF_1508ASV = "ATF1508ASV"

STANDARD_VECTORS = {
    OP_ERASE: {
        ATF_1502: files("atfu.standard_vectors").joinpath("e1502.xsvf"),
        ATF_1504: files("atfu.standard_vectors").joinpath("e1504.xsvf"),
        ATF_1508: files("atfu.standard_vectors").joinpath("e1508.xsvf"),
        ATF_1502AS: files("atfu.standard_vectors").joinpath("e1502.xsvf"),
        ATF_1504AS: files("atfu.standard_vectors").joinpath("e1504.xsvf"),
        ATF_1508AS: files("atfu.standard_vectors").joinpath("e1508.xsvf"),
        ATF_1502ASV: files("atfu.standard_vectors").joinpath("e1502v.xsvf"),
        ATF_1504ASV: files("atfu.standard_vectors").joinpath("e1504v.xsvf"),
        ATF_1508ASV: files("atfu.standard_vectors").joinpath("e1508v.xsvf"),
    },
    OP_CHECK: {
        ATF_1502: files("atfu.standard_vectors").joinpath("bc1502.xsvf"),
        ATF_1504: files("atfu.standard_vectors").joinpath("bc1504.xsvf"),
        ATF_1508: files("atfu.standard_vectors").joinpath("bc1508.xsvf"),
        ATF_1502AS: files("atfu.standard_vectors").joinpath("bc1502.xsvf"),
        ATF_1504AS: files("atfu.standard_vectors").joinpath("bc1504.xsvf"),
        ATF_1508AS: files("atfu.standard_vectors").joinpath("bc1508.xsvf"),
        ATF_1502ASV: files("atfu.standard_vectors").joinpath("bc1502v.xsvf"),
        ATF_1504ASV: files("atfu.standard_vectors").joinpath("bc1504v.xsvf"),
        ATF_1508ASV: files("atfu.standard_vectors").joinpath("bc1508v.xsvf"),
    },
}


def find_vector_file(operation: str, device_key: str):
    return STANDARD_VECTORS.get(operation, {}).get(device_key)



import io
import tempfile
from atfu.output import Output
from atfu.converter.device import device_from_str
from atfu.converter.fuseconv import read_jed, write_svf
from atfu.converter.device import (
    ATF1502ASDevice,
    ATF1504ASDevice,
    ATF1508ASDevice,
    ATF1502ASVDevice,
    ATF1504ASVDevice,
    ATF1508ASVDevice,
)

# TODO don't be exit()ing in here.......


def jed2svf(
    output: Output,
    device: str,
    infile: io.BufferedReader,
    erase: bool,
    verify: bool,
    program: bool,
) -> io.BufferedReader:
    output.output(2, "Converting JED to SVF...")

    # Underlying code expects a text reader, not binary...
    inname = infile.name
    infile.close()
    infile = open(inname, "r")

    c_device = device_from_str(device)
    if c_device is None:
        output.error(
            "Bug",
            f"Incorrect device string {device} passed in jed2svf; Please report: https://github.com/roscopeco/atfprog-tools/issues",
        )
        exit(1000)

    jed_bits = svf_bits = None
    if infile.name.lower().endswith(".jed"):
        jed_bits, comment = read_jed(infile)
        if c_device.fuse_count != len(jed_bits):
            output.error(
                "Failure",
                f"Wrong number of fuses {len(jed_bits)} for device {device} (expected {c_device.fuse_count})",
            )
            exit(50)
    else:
        output.error(
            "Bug",
            "jed2svf called on non-JED; Please report: https://github.com/roscopeco/atfprog-tools/issues",
        )
        exit(1000)

    if svf_bits is None:
        svf_bits = c_device.jed_to_svf(jed_bits)

    outfile = tempfile.NamedTemporaryFile(prefix="atfu-temp-", suffix=".svf", mode="w+")
    write_svf(
        outfile,
        svf_bits,
        c_device,
        comment=comment,
        erase=erase,
        verify=verify,
        program=program,
    )

    outfile.seek(0)
    return outfile



import tempfile

from atfu.output import Output, cbYellow, cReset
from atfu.little_board.jtag_programmer import JtagProgrammer
from atfu.converter.jed2svf import jed2svf
from atfu.converter.svf2xsvf import svf2xsvf
from atfu.converter.fuseconv import write_svf
from atfu.converter.device import device_from_str
from atfu.converter.device import (
    ATF1502ASDevice,
    ATF1504ASDevice,
    ATF1508ASDevice,
    ATF1502ASVDevice,
    ATF1504ASVDevice,
    ATF1508ASVDevice,
)


def handler(args):
    output = Output(args)

    scan_result = scan_device(args, output)

    if scan_result is None:
        if not args.plain:
            output.expected_failure(
                "Scan failed", "Unsupported device, no device or communication error"
            )
        exit(1)
    else:
        if args.plain:
            print(scan_result, end="")
        else:
            output.success(scan_result, "device detected successfully")

        exit(0)


def check_chip_id(args: any, output: Output, device: str = None) -> bool:
    if device is None:
        device = args.device

    c_device = device_from_str(device)
    if c_device is None:
        output.error(
            "Bug",
            f"Incorrect device string {device} passed in jed2svf; Please report: https://github.com/roscopeco/atfprog-tools/issues",
        )
        exit(1000)

    temp_svf = tempfile.NamedTemporaryFile(
        prefix="atfu-temp-", suffix=".svf", mode="w+"
    )

    # Calling without erase, verify or program true will just do device id verification...
    write_svf(
        temp_svf,
        svf_bits=None,
        device=c_device,
        comment="ID check",
        erase=False,
        verify=False,
        program=False,
    )

    temp_svf.seek(0)

    temp_xsvf = svf2xsvf(output=output, infile=temp_svf)

    prog = JtagProgrammer(
        "ID",
        args.programmer,
        verbosity=0,
        no_success=True,
        no_fail=True,
        no_filename=True,
    )

    return prog.upload_all_files([temp_xsvf])


def scan_device(args: any, output: Output) -> str:
    if check_chip_id(args, output, "ATF1502AS"):
        return "ATF1502AS"
    if check_chip_id(args, output, "ATF1504AS"):
        return "ATF1504AS"
    if check_chip_id(args, output, "ATF1508AS"):
        return "ATF1508AS"
    if check_chip_id(args, output, "ATF1502ASV"):
        return "ATF1502ASV"
    if check_chip_id(args, output, "ATF1504ASV"):
        return "ATF1504ASV"
    if check_chip_id(args, output, "ATF1508ASV"):
        return "ATF1508ASV"

    return None
